import { CURSOR_SIZE, BULLET_HOLE_SIZE, BULLET_DURATION } from './Constants.js';
import { Subject } from './Subject';

// Assets
import cursor from '../data/cursor.png';
import bulletHole from '../data/bulletHole.png';
import shot from '../data/shot.mp3';
import empty from '../data/empty.mp3';


class Gun extends Subject {
  constructor(totShots) {
    super();
    this.cursor = null;
    this.shotSound = null;
    this.emptySound = null;

    this.totshots = totShots;
    this.remainingShots = totShots;
    this.bullets = [];  // array of Bullet objects
  }

  setup() {
    this.cursor = loadImage(cursor);
    this.shotSound = loadSound(shot);
    this.emptySound = loadSound(empty);
  }
  draw() {
    noCursor();

    if (this.cursor) {
      image(this.cursor, mouseX - CURSOR_SIZE / 2, mouseY - CURSOR_SIZE / 2, CURSOR_SIZE, CURSOR_SIZE);
    } // custom cursor

    for (let bullet of this.bullets) {
      bullet.draw();
    }   // draw bullet holes

    this.bullets = this.bullets.filter(bullet => bullet.visible); // clean up bullet holes
  }

  shoot() {
    if (this.remainingShots > 0) {
      this.remainingShots--;

      if (this.shotSound) this.shotSound.play();

      const x = mouseX;
      const y = mouseY;

      const bullet = new Bullet(x, y);
      this.bullets.push(bullet);

      this.notifySubscribers('gun', x, y, this.remainingShots);
    } else {
      if (this.emptySound) this.emptySound.play();
    }
  }

  reload() {
    this.remainingShots = this.totShots;
  }

  getRemainingShots() {
    return this.remainingShots;
  }
}


// Bullet
class Bullet {
  constructor(x, y) {
    this.x = x;
    this.y = y;
    this.visible = true;

    this.img = null;
    loadImage(bulletHole, (img) => {
      this.img = img;
    });

    setTimeout(() => {
      this.visible = false;
    }, BULLET_DURATION);
  }

  draw() {
    if (this.visible && this.img) {
      image(this.img, this.x - BULLET_HOLE_SIZE / 2, this.y - BULLET_HOLE_SIZE / 2, BULLET_HOLE_SIZE, BULLET_HOLE_SIZE);
    }
  }
}


export { Gun };
